import pysoem
import threading
import time
import ctypes
import struct
import tkinter as tk
from tkinter import messagebox

# EtherCAT 마스터 생성
master = pysoem.Master()

master.open('eth0')

# 슬레이브 초기화
if master.config_init() > 0:
    print(f'슬레이브 {len(master.slaves)}개 검색됨!')
else:
    print('No slaves found')
    master.close()
    raise Exception("NO SLAVE")

slave = master.slaves[0]

master.config_map()

if master.state_check(pysoem.SAFEOP_STATE, timeout=50_000) != pysoem.SAFEOP_STATE:
    master.close()
    raise Exception("not all slaves reached SAFEOP state")
        
slave.dc_sync(act=True, sync0_cycle_time=10_000_000)
master.state = pysoem.OP_STATE

master.send_processdata()
master.receive_processdata(timeout=2000)
# request OP state for all slaves
master.write_state()
all_slaves_reached_op_state = False
for i in range(40):
    master.state_check(pysoem.OP_STATE, timeout=50_000)
    if master.state == pysoem.OP_STATE:
        all_slaves_reached_op_state = True
        break
if all_slaves_reached_op_state:
    print("모든 슬레이브 활성상태")


# GUI 설정
root = tk.Tk()
root.title("녹칸다의 이더캣통신 프로그램!")

root.geometry("600x400")

label1 = tk.Label(root, text="A0 측정값=0", font=("Arial", 30))
label1.pack()

def thread1():
    while True:
        master.send_processdata()
        master.receive_processdata(timeout=2000)
        cnt1 = int.from_bytes(slave.input[0:2], byteorder='little', signed=False) 
        #print(f"cnt1={cnt1}")
        label1.config(text=f"A0 측정값={cnt1}")
        time.sleep(0.1)

ethercat_thread = threading.Thread(target=thread1, daemon=True)
ethercat_thread.start()

root.mainloop()

print("프로그램이 종료되었습니다.")
master.state = pysoem.INIT_STATE
# request INIT state for all slaves
master.write_state()
master.close()


